@@ -35,6 +35,50 @@
 #include "path.h"
 #include "util.h"
 
+/** This enum defines the states of the SAX parsing state machine. */
+typedef enum
+{
+    START,
+    INSIDE_GPX,
+    INSIDE_WPT,
+    INSIDE_WPT_NAME,
+    INSIDE_WPT_DESC,
+    INSIDE_WPT_CMT,
+    INSIDE_PATH,
+    INSIDE_PATH_SEGMENT,
+    INSIDE_PATH_POINT,
+    INSIDE_PATH_POINT_ELE,
+    INSIDE_PATH_POINT_TIME,
+    INSIDE_PATH_POINT_DESC,
+    INSIDE_PATH_POINT_CMT,
+    FINISH,
+    UNKNOWN,
+    ERROR,
+} SaxState;
+
+/** Data used during the SAX parsing operation. */
+typedef struct _SaxData SaxData;
+struct _SaxData {
+    SaxState state;
+    SaxState prev_state;
+    gint unknown_depth;
+    gboolean at_least_one_trkpt;
+    GString *chars;
+};
+
+typedef struct _PathSaxData PathSaxData;
+struct _PathSaxData {
+    SaxData sax_data;
+    Path path;
+};
+
+typedef struct _PoiSaxData PoiSaxData;
+struct _PoiSaxData {
+    SaxData sax_data;
+    GList *poi_list;
+    PoiInfo *curr_poi;
+};
+
 /**
  * Handle a start tag in the parsing of a GPX file.
  */
@@ -65,6 +109,7 @@ gpx_chars(SaxData *data, const xmlChar *ch, int len)
         case INSIDE_PATH_POINT_ELE:
         case INSIDE_PATH_POINT_TIME:
         case INSIDE_PATH_POINT_DESC:
+        case INSIDE_PATH_POINT_CMT:
             for(i = 0; i < len; i++)
                 data->chars = g_string_append_c(data->chars, ch[i]);
             vprintf("%s\n", data->chars->str);
@@ -99,6 +144,81 @@ gpx_error(SaxData *data, const gchar *msg, ...)
     data->state = ERROR;
 }
 
+static gboolean
+gpx_write_string(GnomeVFSHandle *handle, const gchar *str)
+{
+    GnomeVFSResult vfs_result;
+    GnomeVFSFileSize size;
+    if(GNOME_VFS_OK != (vfs_result = gnome_vfs_write(
+                    handle, str, strlen(str), &size)))
+    {
+        gchar buffer[BUFFER_SIZE];
+        snprintf(buffer, sizeof(buffer),
+                "%s:\n%s\n%s", _("Error while writing to file"),
+                _("File is incomplete."),
+                gnome_vfs_result_to_string(vfs_result));
+        popup_error(_window, buffer);
+        return FALSE;
+    }
+    return TRUE;
+}
+
+static gboolean
+gpx_write_escaped(GnomeVFSHandle *handle, const gchar *str)
+{
+    const gchar *ptr = str;
+    const gchar *nullchr = ptr + strlen(ptr);
+    while(ptr < nullchr)
+    {
+        gchar *newptr = strpbrk(ptr, "&<>");
+        if(newptr != NULL)
+        {
+            /* First, write out what we have so far. */
+            const gchar *to_write;
+            GnomeVFSResult vfs_result;
+            GnomeVFSFileSize size;
+            if(GNOME_VFS_OK != (vfs_result = gnome_vfs_write(
+                            handle, ptr, newptr - ptr, &size)))
+            {
+                gchar buffer[BUFFER_SIZE];
+                snprintf(buffer, sizeof(buffer),
+                        "%s:\n%s\n%s", _("Error while writing to file"),
+                        _("File is incomplete."),
+                        gnome_vfs_result_to_string(vfs_result));
+                popup_error(_window, buffer);
+                return FALSE;
+            }
+
+            /* Now, write the XML entity. */
+            switch(*newptr)
+            {
+                case '&':
+                    to_write = "&amp;";
+                    break;
+                case '<':
+                    to_write = "&lt;";
+                    break;
+                case '>':
+                    to_write = "&gt;";
+                    break;
+                default:
+                    to_write = "";
+            }
+            gpx_write_string(handle, to_write);
+
+            /* Advance the pointer to continue searching for entities. */
+            ptr = newptr + 1;
+        }
+        else
+        {
+            /* No characters need escaping - write the whole thing. */
+            gpx_write_string(handle, ptr);
+            ptr = nullchr;
+        }
+    }
+    return TRUE;
+}
+
 /****************************************************************************
  * BELOW: OPEN PATH *********************************************************
  ****************************************************************************/
@@ -118,13 +238,17 @@ gpx_path_start_element(PathSaxData *data,
             if(!strcmp((gchar*)name, "gpx"))
                 data->sax_data.state = INSIDE_GPX;
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case INSIDE_GPX:
             if(!strcmp((gchar*)name, "trk"))
                 data->sax_data.state = INSIDE_PATH;
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case INSIDE_PATH:
             if(!strcmp((gchar*)name, "trkseg"))
@@ -133,7 +257,9 @@ gpx_path_start_element(PathSaxData *data,
                 data->sax_data.at_least_one_trkpt = FALSE;
             }
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case INSIDE_PATH_SEGMENT:
             if(!strcmp((gchar*)name, "trkpt"))
@@ -175,7 +301,9 @@ gpx_path_start_element(PathSaxData *data,
                     data->sax_data.state = ERROR;
             }
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case INSIDE_PATH_POINT:
             if(!strcmp((gchar*)name, "time"))
@@ -184,9 +312,13 @@ gpx_path_start_element(PathSaxData *data,
                 data->sax_data.state = INSIDE_PATH_POINT_ELE;
             else if(!strcmp((gchar*)name, "desc"))
                 data->sax_data.state = INSIDE_PATH_POINT_DESC;
+            else if(!strcmp((gchar*)name, "cmt"))
+                data->sax_data.state = INSIDE_PATH_POINT_CMT;
 
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case UNKNOWN:
             data->sax_data.unknown_depth++;
@@ -327,11 +459,39 @@ gpx_path_end_element(PathSaxData *data, const xmlChar *name)
             else
                 data->sax_data.state = ERROR;
             break;
+        case INSIDE_PATH_POINT_CMT:
+            /* only parse description for routes */
+            if(!strcmp((gchar*)name, "cmt"))
+            {
+                if(data->path.wtail < data->path.whead
+                        || data->path.wtail->point != data->path.tail)
+                {
+                    MACRO_PATH_INCREMENT_WTAIL(data->path);
+                    data->path.wtail->point = data->path.tail;
+                    data->path.wtail->desc
+                        = g_string_free(data->sax_data.chars, FALSE);
+                }
+                else
+                    g_string_free(data->sax_data.chars, TRUE);
+
+                data->sax_data.chars = g_string_new("");
+                data->sax_data.state = INSIDE_PATH_POINT;
+            }
+            else
+                data->sax_data.state = ERROR;
+            break;
         case INSIDE_PATH_POINT_DESC:
             /* only parse description for routes */
             if(!strcmp((gchar*)name, "desc"))
             {
-                MACRO_PATH_INCREMENT_WTAIL(data->path);
+                /* If we already have a desc (e.g. from cmt), then overwrite */
+                if(data->path.wtail >= data->path.whead
+                        && data->path.wtail->point == data->path.tail)
+                    g_free(data->path.wtail->desc);
+                else
+                {
+                    MACRO_PATH_INCREMENT_WTAIL(data->path);
+                }
                 data->path.wtail->point = data->path.tail;
                 data->path.wtail->desc
                     = g_string_free(data->sax_data.chars, FALSE);
@@ -344,8 +504,6 @@ gpx_path_end_element(PathSaxData *data, const xmlChar *name)
         case UNKNOWN:
             if(!--data->sax_data.unknown_depth)
                 data->sax_data.state = data->sax_data.prev_state;
-            else
-                data->sax_data.state = ERROR;
             break;
         default:
             ;
@@ -470,22 +628,6 @@ gpx_path_parse(Path *to_replace, gchar *buffer, gint size, gint policy_old)
  * BELOW: SAVE PATH *********************************************************
  ****************************************************************************/
 
-#define WRITE_STRING(string) { \
-    GnomeVFSResult vfs_result; \
-    GnomeVFSFileSize size; \
-    if(GNOME_VFS_OK != (vfs_result = gnome_vfs_write( \
-                    handle, (string), strlen((string)), &size))) \
-    { \
-        gchar buffer[BUFFER_SIZE]; \
-        snprintf(buffer, sizeof(buffer), \
-                "%s:\n%s\n%s", _("Error while writing to file"), \
-                _("File is incomplete."), \
-                gnome_vfs_result_to_string(vfs_result)); \
-        popup_error(_window, buffer); \
-        return FALSE; \
-    } \
-}
-
 gboolean
 gpx_path_write(Path *path, GnomeVFSHandle *handle)
 {
@@ -504,7 +646,7 @@ gpx_path_write(Path *path, GnomeVFSHandle *handle)
     }
 
     /* Write the header. */
-    WRITE_STRING(
+    gpx_write_string(handle,
             "<?xml version=\"1.0\"?>\n"
             "<gpx version=\"1.0\" creator=\"maemo-mapper\" "
             "xmlns=\"http://www.topografix.com/GPX/1/0\">\n"
@@ -522,33 +664,33 @@ gpx_path_write(Path *path, GnomeVFSHandle *handle)
             if(trkseg_break)
             {
                 /* First trkpt of the segment - write trkseg header. */
-                WRITE_STRING("    </trkseg>\n"
+                gpx_write_string(handle, "    </trkseg>\n"
                              "    <trkseg>\n");
                 trkseg_break = FALSE;
             }
             unit2latlon(curr->unitx, curr->unity, lat, lon);
-            WRITE_STRING("      <trkpt lat=\"");
+            gpx_write_string(handle, "      <trkpt lat=\"");
             g_ascii_formatd(buffer, sizeof(buffer), "%.06f", lat);
-            WRITE_STRING(buffer);
-            WRITE_STRING("\" lon=\"");
+            gpx_write_string(handle, buffer);
+            gpx_write_string(handle, "\" lon=\"");
             g_ascii_formatd(buffer, sizeof(buffer), "%.06f", lon);
-            WRITE_STRING(buffer);
-            WRITE_STRING("\"");
+            gpx_write_string(handle, buffer);
+            gpx_write_string(handle, "\"");
 
             /* write the elevation */
             if(curr->altitude != 0)
             {
                 if(first_sub)
                 {
-                    WRITE_STRING(">\n");
+                    gpx_write_string(handle, ">\n");
                     first_sub = FALSE;
                 }
-                WRITE_STRING("        <ele>");
+                gpx_write_string(handle, "        <ele>");
                 {
                     g_ascii_formatd(buffer, 80, "%.2f", curr->altitude);
-                    WRITE_STRING(buffer);
+                    gpx_write_string(handle, buffer);
                 }
-                WRITE_STRING("</ele>\n");
+                gpx_write_string(handle, "</ele>\n");
             }
 
             /* write the time */
@@ -556,35 +698,35 @@ gpx_path_write(Path *path, GnomeVFSHandle *handle)
             {
                 if(first_sub)
                 {
-                    WRITE_STRING(">\n");
+                    gpx_write_string(handle, ">\n");
                     first_sub = FALSE;
                 }
-                WRITE_STRING("        <time>");
+                gpx_write_string(handle, "        <time>");
                 strftime(buffer, 80, XML_DATE_FORMAT, localtime(&curr->time));
-                WRITE_STRING(buffer);
-                WRITE_STRING(XML_TZONE);
-                WRITE_STRING("</time>\n");
+                gpx_write_string(handle, buffer);
+                gpx_write_string(handle, XML_TZONE);
+                gpx_write_string(handle, "</time>\n");
             }
 
             if(wcurr && curr == wcurr->point)
             {
                 if(first_sub)
                 {
-                    WRITE_STRING(">\n");
+                    gpx_write_string(handle, ">\n");
                     first_sub = FALSE;
                 }
-                WRITE_STRING("        <desc>");
-                WRITE_STRING(wcurr->desc);
-                WRITE_STRING("</desc>\n");
+                gpx_write_string(handle, "        <desc>");
+                gpx_write_escaped(handle, wcurr->desc);
+                gpx_write_string(handle, "</desc>\n");
                 wcurr++;
             }
             if(first_sub)
             {
-                WRITE_STRING("/>\n");
+                gpx_write_string(handle, "/>\n");
             }
             else
             {
-                WRITE_STRING("      </trkpt>\n");
+                gpx_write_string(handle, "      </trkpt>\n");
             }
         }
         else
@@ -592,7 +734,7 @@ gpx_path_write(Path *path, GnomeVFSHandle *handle)
     }
 
     /* Write the footer. */
-    WRITE_STRING(
+    gpx_write_string(handle,
             "    </trkseg>\n"
             "  </trk>\n"
             "</gpx>\n");
@@ -625,7 +767,9 @@ gpx_poi_start_element(PoiSaxData *data,
             if(!strcmp((gchar*)name, "gpx"))
                 data->sax_data.state = INSIDE_GPX;
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case INSIDE_GPX:
             if(!strcmp((gchar*)name, "wpt"))
@@ -668,18 +812,23 @@ gpx_poi_start_element(PoiSaxData *data,
                     data->sax_data.state = ERROR;
             }
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case INSIDE_WPT:
             if(!strcmp((gchar*)name, "name"))
                 data->sax_data.state = INSIDE_WPT_NAME;
             else if(!strcmp((gchar*)name, "desc"))
                 data->sax_data.state = INSIDE_WPT_DESC;
+            else if(!strcmp((gchar*)name, "cmt"))
+                data->sax_data.state = INSIDE_WPT_CMT;
             else
+            {
                 MACRO_SET_UNKNOWN();
+            }
             break;
         case UNKNOWN:
-            printf("UNKNOWN!\n");
             data->sax_data.unknown_depth++;
             break;
         default:
@@ -730,9 +879,27 @@ gpx_poi_end_element(PoiSaxData *data, const xmlChar *name)
             else
                 data->sax_data.state = ERROR;
             break;
+        case INSIDE_WPT_CMT:
+            if(!strcmp((gchar*)name, "cmt"))
+            {
+                /* Only use if we don't already have a desc */
+                if(!data->curr_poi->desc)
+                {
+                    data->curr_poi->desc
+                        = g_string_free(data->sax_data.chars, FALSE);
+                    data->sax_data.chars = g_string_new("");
+                    data->sax_data.state = INSIDE_WPT;
+                }
+            }
+            else
+                data->sax_data.state = ERROR;
+            break;
         case INSIDE_WPT_DESC:
             if(!strcmp((gchar*)name, "desc"))
             {
+                /* If we already have a desc (e.g. from cmt), then overwrite */
+                if(data->curr_poi->desc)
+                    g_free(data->curr_poi->desc);
                 data->curr_poi->desc
                     = g_string_free(data->sax_data.chars, FALSE);
                 data->sax_data.chars = g_string_new("");
@@ -742,11 +909,8 @@ gpx_poi_end_element(PoiSaxData *data, const xmlChar *name)
                 data->sax_data.state = ERROR;
             break;
         case UNKNOWN:
-            printf("UNKNOWN!\n");
             if(!--data->sax_data.unknown_depth)
                 data->sax_data.state = data->sax_data.prev_state;
-            else
-                data->sax_data.state = ERROR;
             break;
         default:
             ;
@@ -797,22 +961,6 @@ gpx_poi_parse(gchar *buffer, gint size, GList **poi_list)
  * BELOW: SAVE POI **********************************************************
  ****************************************************************************/
 
-#define WRITE_STRING(string) { \
-    GnomeVFSResult vfs_result; \
-    GnomeVFSFileSize size; \
-    if(GNOME_VFS_OK != (vfs_result = gnome_vfs_write( \
-                    handle, (string), strlen((string)), &size))) \
-    { \
-        gchar buffer[BUFFER_SIZE]; \
-        snprintf(buffer, sizeof(buffer), \
-                "%s:\n%s\n%s", _("Error while writing to file"), \
-                _("File is incomplete."), \
-                gnome_vfs_result_to_string(vfs_result)); \
-        popup_error(_window, buffer); \
-        return FALSE; \
-    } \
-}
-
 gint
 gpx_poi_write(GtkTreeModel *model, GnomeVFSHandle *handle)
 {
@@ -821,7 +969,7 @@ gpx_poi_write(GtkTreeModel *model, GnomeVFSHandle *handle)
     printf("%s()\n", __PRETTY_FUNCTION__);
 
     /* Write the header. */
-    WRITE_STRING(
+    gpx_write_string(handle,
             "<?xml version=\"1.0\"?>\n"
             "<gpx version=\"1.0\" creator=\"maemo-mapper\" "
             "xmlns=\"http://www.topografix.com/GPX/1/0\">\n");
@@ -848,35 +996,34 @@ gpx_poi_write(GtkTreeModel *model, GnomeVFSHandle *handle)
         {
             gchar buffer[80];
 
-            WRITE_STRING("  <wpt lat=\"");
+            gpx_write_string(handle, "  <wpt lat=\"");
             g_ascii_formatd(buffer, sizeof(buffer), "%.06f", poi.lat);
-            WRITE_STRING(buffer);
-            WRITE_STRING("\" lon=\"");
+            gpx_write_string(handle, buffer);
+            gpx_write_string(handle, "\" lon=\"");
             g_ascii_formatd(buffer, sizeof(buffer), "%.06f", poi.lon);
-            WRITE_STRING(buffer);
-            WRITE_STRING("\"/>\n");
+            gpx_write_string(handle, buffer);
+            gpx_write_string(handle, "\">\n");
 
             if(poi.label && *poi.label)
             {
-                WRITE_STRING("    <name>");
-                WRITE_STRING(poi.label);
-                WRITE_STRING("</name>\n");
+                gpx_write_string(handle, "    <name>");
+                gpx_write_escaped(handle, poi.label);
+                gpx_write_string(handle, "</name>\n");
             }
 
             if(poi.desc && *poi.desc)
             {
-                WRITE_STRING("    <desc>");
-                WRITE_STRING(poi.desc);
-                WRITE_STRING("</desc>\n");
+                gpx_write_string(handle, "    <desc>");
+                gpx_write_escaped(handle, poi.desc);
+                gpx_write_string(handle, "</desc>\n");
             }
-            WRITE_STRING("  </wpt>\n");
+            gpx_write_string(handle, "  </wpt>\n");
             ++ num_written;
         }
     } while(gtk_tree_model_iter_next(model, &iter));
 
     /* Write the footer. */
-    WRITE_STRING(
-            "</gpx>\n");
+    gpx_write_string(handle, "</gpx>\n");
 
     vprintf("%s(): return %d\n", __PRETTY_FUNCTION__, num_written);
     return num_written;
