@@ -34,9 +34,15 @@
 #include <locale.h>
 
 #include <gconf/gconf-client.h>
-#include <device_symbols.h>
-#include <conicconnection.h>
-#include <conicconnectionevent.h>
+
+#ifdef MAEMO_CHANGES /* probably not the best macro to check for here */
+#    include <device_symbols.h>
+#endif
+
+#ifdef CONIC
+#    include <conicconnection.h>
+#    include <conicconnectionevent.h>
+#endif
 
 #ifndef LEGACY
 #    include <hildon/hildon-program.h>
@@ -68,16 +74,19 @@ static void osso_cb_hw_state(osso_hw_state_t *state, gpointer data);
 
 static HildonProgram *_program = NULL;
 
+#ifdef CONIC
 static ConIcConnection *_conic_conn = NULL;
 static gboolean _conic_is_connecting = FALSE;
 static gboolean _conic_conn_failed = FALSE;
 static GMutex *_conic_connection_mutex = NULL;
 static GCond *_conic_connection_cond = NULL;
+#endif
 
 /* Dynamically-sized in-memory map cache. */
 static size_t _map_cache_size = (32*1024*1024);
 static gboolean _map_cache_enabled = TRUE;
 
+#ifdef CONIC
 static void
 conic_conn_event(ConIcConnection *connection, ConIcConnectionEvent *event)
 {
@@ -88,7 +97,7 @@ conic_conn_event(ConIcConnection *connection, ConIcConnectionEvent *event)
 
     status = con_ic_connection_event_get_status(event);
 
-    if((_conic_is_connected = (status == CON_IC_CONNECTION_ERROR_NONE)))
+    if((_conic_is_connected = (status == CON_IC_STATUS_CONNECTED)))
     {
         /* We're connected. */
         _conic_conn_failed = FALSE;
@@ -110,13 +119,14 @@ conic_conn_event(ConIcConnection *connection, ConIcConnectionEvent *event)
 
     vprintf("%s(): return\n", __PRETTY_FUNCTION__);
 }
+#endif
 
 void
 conic_recommend_connected()
 {
     printf("%s()\n", __PRETTY_FUNCTION__);
 
-#ifndef DEBUG
+#if defined(__arm__) && defined(CONIC)
     g_mutex_lock(_conic_connection_mutex);
     if(!_conic_is_connecting)
     {
@@ -135,7 +145,7 @@ conic_ensure_connected()
 {
     printf("%s()\n", __PRETTY_FUNCTION__);
 
-#ifndef DEBUG
+#if defined(__arm__) && defined(CONIC)
     while(_window && !_conic_is_connected)
     {   
         g_mutex_lock(_conic_connection_mutex);
@@ -150,6 +160,8 @@ conic_ensure_connected()
         g_cond_wait(_conic_connection_cond, _conic_connection_mutex);
         g_mutex_unlock(_conic_connection_mutex);
     }
+#else
+    _conic_is_connected = TRUE;
 #endif
 
     vprintf("%s(): return %d\n", __PRETTY_FUNCTION__,
@@ -182,13 +194,16 @@ maemo_mapper_destroy()
     g_mutex_unlock(_mut_priority_mutex);
 
     /* Allow remaining downloads to finish. */
+#ifdef CONIC
     g_mutex_lock(_conic_connection_mutex);
     g_cond_broadcast(_conic_connection_cond);
     g_mutex_unlock(_conic_connection_mutex);
+#endif
     g_thread_pool_free(_mut_thread_pool, TRUE, TRUE);
 
     if(_curr_repo->db)
     {
+        RepoData* repo_p;
 #ifdef MAPDB_SQLITE
         g_mutex_lock(_mapdb_mutex);
         sqlite3_close(_curr_repo->db);
@@ -196,8 +211,17 @@ maemo_mapper_destroy()
         g_mutex_unlock(_mapdb_mutex);
 #else
         g_mutex_lock(_mapdb_mutex);
-        gdbm_close(_curr_repo->db);
-        _curr_repo->db = NULL;
+        repo_p = _curr_repo;
+        while (repo_p) {
+            if (repo_p->db) {
+/*                 /\* perform reorganization for layers which are auto refreshed *\/ */
+/*                 if (repo_p->layer_level && repo_p->layer_refresh_interval) */
+/*                     gdbm_reorganize (repo_p->db); */
+                gdbm_close(repo_p->db);
+            }
+            repo_p->db = NULL;
+            repo_p = repo_p->layers;
+        }
         g_mutex_unlock(_mapdb_mutex);
 #endif
     }
@@ -241,6 +265,7 @@ maemo_mapper_init(gint argc, gchar **argv)
     INFO_FONT_ENUM_TEXT[INFO_FONT_XLARGE] = "x-large";
     INFO_FONT_ENUM_TEXT[INFO_FONT_XXLARGE] = "xx-large";
 
+#ifdef MAEMO_CHANGES /* probably not the best macro to check for here */
     CUSTOM_KEY_ICON[CUSTOM_KEY_UP] = HWK_BUTTON_UP;
     CUSTOM_KEY_ICON[CUSTOM_KEY_LEFT] = HWK_BUTTON_LEFT;
     CUSTOM_KEY_ICON[CUSTOM_KEY_DOWN] = HWK_BUTTON_DOWN;
@@ -250,6 +275,17 @@ maemo_mapper_init(gint argc, gchar **argv)
     CUSTOM_KEY_ICON[CUSTOM_KEY_DECREASE] = HWK_BUTTON_DECREASE;
     CUSTOM_KEY_ICON[CUSTOM_KEY_FULLSCREEN] = HWK_BUTTON_VIEW;
     CUSTOM_KEY_ICON[CUSTOM_KEY_ESC] = HWK_BUTTON_CANCEL;
+#else
+    CUSTOM_KEY_ICON[CUSTOM_KEY_UP] = "Up";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_LEFT] = "Left";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_DOWN] = "Down";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_RIGHT] = "Right";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_SELECT] = "Enter";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_INCREASE] = "F7";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_DECREASE] = "F8";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_FULLSCREEN] = "F6";
+    CUSTOM_KEY_ICON[CUSTOM_KEY_ESC] = "Esc";
+#endif
 
     CUSTOM_KEY_DEFAULT[CUSTOM_KEY_UP] = CUSTOM_ACTION_RESET_VIEW_ANGLE;
     CUSTOM_KEY_DEFAULT[CUSTOM_KEY_LEFT] =CUSTOM_ACTION_ROTATE_COUNTERCLOCKWISE;
@@ -283,7 +319,10 @@ maemo_mapper_init(gint argc, gchar **argv)
         = _("Toggle Fullscreen");
     CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_ZOOM_IN] = _("Zoom In");
     CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_ZOOM_OUT] = _("Zoom Out");
-    CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_TOGGLE_TRACKS] = _("Toggle Tracks");
+    CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_TOGGLE_TRACKING]
+        = _("Toggle Tracking");
+    CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_TOGGLE_TRACKS]
+        = _("Toggle Tracks/Routes");
     CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_TOGGLE_SCALE] = _("Toggle Scale");
     CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_TOGGLE_POI] = _("Toggle POIs");
     CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_CHANGE_REPO]
@@ -304,6 +343,7 @@ maemo_mapper_init(gint argc, gchar **argv)
         = _("Toggle Speed Limit");
     CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_RESET_BLUETOOTH]
         = _("Reset Bluetooth");
+    CUSTOM_ACTION_ENUM_TEXT[CUSTOM_ACTION_TOGGLE_LAYERS] = _("Toggle Layers");
 
     DEG_FORMAT_ENUM_TEXT[DDPDDDDD] = "-dd.ddddd°";
     DEG_FORMAT_ENUM_TEXT[DD_MMPMMM] = "-dd°mm.mmm'";
@@ -327,15 +367,16 @@ maemo_mapper_init(gint argc, gchar **argv)
     /* Set up track array (must be done before config). */
     memset(&_track, 0, sizeof(_track));
     memset(&_route, 0, sizeof(_route));
-    MACRO_PATH_INIT(_track);
-    MACRO_PATH_INIT(_route);
+    /* initialisation of paths is done in path_init() */
 
     _mapdb_mutex = g_mutex_new();
     _mut_priority_mutex = g_mutex_new();
     _mouse_mutex = g_mutex_new();
 
+#ifdef CONIC
     _conic_connection_mutex = g_mutex_new();
     _conic_connection_cond = g_cond_new();
+#endif
 
     settings_init();
     map_cache_init(_map_cache_size);
@@ -348,6 +389,8 @@ maemo_mapper_init(gint argc, gchar **argv)
     _window = GTK_WIDGET(hildon_window_new());
     hildon_program_add_window(_program, HILDON_WINDOW(_window));
 
+    gtk_window_set_default_size(GTK_WINDOW(_window), 800, 480);
+
     /* Lets go fullscreen if so requested in saved config */
     if (_fullscreen) {
       gtk_window_fullscreen(GTK_WINDOW(_window));
@@ -508,10 +551,12 @@ maemo_mapper_init(gint argc, gchar **argv)
     /* If we have a route, calculate the next point. */
     route_find_nearest_point();
 
+#ifdef CONIC
     _conic_conn = con_ic_connection_new();
     g_object_set(_conic_conn, "automatic-connection-events", TRUE, NULL);
     g_signal_connect(G_OBJECT(_conic_conn), "connection-event",
             G_CALLBACK(conic_conn_event), NULL);
+#endif
 
     g_idle_add((GSourceFunc)window_present, NULL);
 
